#pragma once

#include "ValWrapper.h"
#include "VariableInterfaces.h"
#include "IExpIndicator.h"

class CIntrisincsWrappers
{
public:
	explicit CIntrisincsWrappers(intrinsic_functions::IntrinsicCtx& _ctx)
		: m_ctx(_ctx)
	{}
private:
	template<class TSimple>
	class CObjWrapper
	{
	public:
		CObjWrapper(_MRetVal<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(IVariable<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(_ELAPI_::user_vars::CStackVar<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(_ELAPI_::user_vars::CSimpleVarObjectBase<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(_ELAPI_::user_vars::CSeriesVarObjectBase<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(_InputVal<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(const CInputObjectBase<TSimple>& _obj)
			:	m_Obj(_obj.get_rebinded_input()), m_bb(_obj.get_rebinded_input_bb())
		{}
	public:
		_InputVal<TSimple>&	get_object()const{return m_Obj;}
		real				get_bb()const{return m_bb;}
	private:
		_InputVal<TSimple>&	m_Obj;
		const real			m_bb;
	};

	template<class TSimple>
	class CLiteralParamWrapper
	{
	public:
		CLiteralParamWrapper(const TSimple& _val)
			:	Value(_val)
		{}

		CLiteralParamWrapper(_MRetVal<TSimple>& _obj)
			:	Value(_obj.Value())
		{}

		CLiteralParamWrapper(IVariable<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CLiteralParamWrapper(_ELAPI_::user_vars::CStackVar<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CLiteralParamWrapper(_ELAPI_::user_vars::CSimpleVarObjectBase<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CLiteralParamWrapper(_ELAPI_::user_vars::CSeriesVarObjectBase<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CLiteralParamWrapper(_InputVal<TSimple>& _obj)
			:	Value(_obj.Value())
		{}

		CLiteralParamWrapper(CInputObjectBase<TSimple>& _obj)
			:	Value(_obj.Value())
		{}
	public:
		const TSimple	Value;
	};

	template<class TSimple>
	class CRefParamWrapper
	{
	public:
		CRefParamWrapper(TSimple& _val)
			:	Value(_val)
		{}

		CRefParamWrapper(_MRetVal<TSimple>& _obj)
			:	Value(_obj.Value())
		{}

		CRefParamWrapper(IVariable<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CRefParamWrapper(_ELAPI_::user_vars::CStackVar<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CRefParamWrapper(_ELAPI_::user_vars::CSimpleVarObjectBase<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CRefParamWrapper(_ELAPI_::user_vars::CSeriesVarObjectBase<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CRefParamWrapper(_InputVal<TSimple>& _obj)
			:	Value(_obj.Value())
		{}

		CRefParamWrapper(CInputObjectBase<TSimple>& _obj)
			:	Value(_obj.Value())
		{}
	public:
		TSimple&	Value;
	};

public:
	NumSimple	intrinsic_summation(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).summ(m_ctx,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_average(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return intrinsic_summation(_obj,_lenght)/_lenght.Value;
	}

	void		intrinsic_extremes(	const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght, 
									const CLiteralParamWrapper<NumSimple>& _min_or_max/*-1 - min, 1 - max*/,
									const CRefParamWrapper<NumSimple>& _result, const CRefParamWrapper<NumSimple>& _bb_offset)
	{
		get_interface( _obj.get_object() ).extremes(m_ctx,_min_or_max.Value,_obj.get_bb(),
													_lenght.Value,_result.Value,_bb_offset.Value);
	}

	NumSimple	intrinsic_waverage(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).waverage(m_ctx,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_varianceps(const CObjWrapper<NumSimple>& _obj,
									 const CLiteralParamWrapper<NumSimple>& _lenght,
									 const CLiteralParamWrapper<NumSimple>& _data_type)
	{
		return get_interface( _obj.get_object() ).varianceps(m_ctx,_data_type.Value,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_ulcerindex(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).ulcerindex(m_ctx,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_summationif(	const CObjWrapper<NumSimple>& _obj,
										const CObjWrapper<BoolSimple>& _test,
										const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).summif(m_ctx,_test.get_object(),_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_pivot(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght,
								const CLiteralParamWrapper<NumSimple>& _leftstrength,
								const CLiteralParamWrapper<NumSimple>& _rightstrength,
								const CLiteralParamWrapper<NumSimple>& _instance,
								const CLiteralParamWrapper<NumSimple>& _min_or_max/*-1 - min, 1 - max*/,
								const CRefParamWrapper<NumSimple>& _oPivotprice,
								const CRefParamWrapper<NumSimple>& _oPivot_bb_offset)
	{
		return get_interface( _obj.get_object() ).pivot(m_ctx,_obj.get_bb(),_lenght.Value,_min_or_max.Value,
					_leftstrength.Value,_rightstrength.Value,_instance.Value,_oPivotprice.Value,_oPivot_bb_offset.Value);
	}

	NumSimple	intrinsic_linearreg(const CObjWrapper<NumSimple>& _obj,
									const CLiteralParamWrapper<NumSimple>& _lenght,
									const CLiteralParamWrapper<NumSimple>& _tgt_bar,
									const CRefParamWrapper<NumSimple>& _oLR_Slope,
									const CRefParamWrapper<NumSimple>& _oLR_Angle,
									const CRefParamWrapper<NumSimple>& _oLR_Intercept,
									const CRefParamWrapper<NumSimple>& _oLR_Value)
	{
		return get_interface( _obj.get_object() ).linear_reg(m_ctx,_obj.get_bb(),_lenght.Value,
											_tgt_bar.Value,_oLR_Slope.Value,_oLR_Angle.Value,_oLR_Intercept.Value,_oLR_Value.Value);
	}

	NumSimple	intrinsic_stderror(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).std_error(m_ctx,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_countif(const CObjWrapper<BoolSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).count_if(m_ctx,_obj.get_bb(),_lenght.Value);
	}

private:
	template<class TSimple>
	const intrinsic_functions::IntrinsicFunctions<TSimple>&	get_interface(_InputVal<TSimple>& _obj)
	{
		const intrinsic_functions::IntrinsicFunctions<TSimple>*const _ptr=_obj.GetIntrinsics();
		return _ptr ? *_ptr : m_ctx.base_study().GetIntrinsicAccessor().GetIntrinsics(_obj);
	}
private:
	intrinsic_functions::IntrinsicCtx&	m_ctx;
};